function Test3(hdr_rate,pam_levels,print_plot)
% Simulation of ACT transmit and receive signals - Test 3
% Function arguments:
%   Test3(hdr_rate,pam_levels,print_plot)
% where hdr_rate is the data rate in the high data rate direction, given 
% in Gbps (default is 2.5, for 2.5Gbps). The pam_levels is the number of 
% levels in the high data rate PAM modulation (default is 4, for PAM4). 
% The print_plot parameter controls if the the plots generated by the 
% test code should be saved to an image file (default is 0, for not 
% saving the image files).
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This is simulation code provided to help with the development of
% IEEE 802.3dm.
%
% This code is provided for reference to allow independent evaluation
% of the accuracy and applicability of the simulation results shared in
% IEEE 802.3dm presentations by the author.
%
% Written by Ragnar Jonsson, affiliated with Marvell Technology, Inc.
% Version 1.1, January 20th, 2025
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
% OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
% THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
% DEALINGS IN THE SOFTWARE.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    %%% Test if MATLAB or Octave %%%
    is_octave = exist('OCTAVE_VERSION', 'builtin') ~= 0;

    %%% Default arguments %%%
    if((nargin < 1) || isempty(hdr_rate))
        hdr_rate = 2.5; % Set default high data rate to 2.5 Gbps
    end
    if((nargin < 2) || isempty(pam_levels))
        pam_levels = 4; % Set default PAM levels to 4 (PAM4)
    end
    if((nargin < 3) || isempty(print_plot))
        print_plot = 0; % Set default to not save plots
    end

    %%% Check data rate %%%
    switch(hdr_rate)
        case 2.5
            hdr_rate_text = '2.5'; % Text for 2.5 Gbps
        case 5
            hdr_rate_text = '5'; % Text for 5 Gbps
        case 10
            hdr_rate_text = '10'; % Text for 10 Gbps
        otherwise
            error(sprintf('Invalid data rate selected %2.2fGbps',hdr_rate)); % Error for invalid data rate
    end

    %%% Load hybrid model %%%
    load jonsson_e241216b_hybrid_echo.mat % Load hybrid echo model

    %%% Load "good" and "bad" cable from jonsson_3dm_02_09_15_24 %%%
    load cable_jonsson_3dm_02_09_15_24_good.mat % Load good cable model
    cable_good = cable; % Assign to cable_good
    load cable_jonsson_3dm_02_09_15_24_bad.mat % Load bad cable model
    cable_bad = cable; % Assign to cable_bad
    is_coax = cable.is_coax; % Check if cable is coaxial

    %%% Configure simulation variables %%%
    ldr_symbols = 100; % Number of low data rate symbols
    t = [0:(192*ldr_symbols)-1]/cable.F_s; % Time vector

    %%% Create test case name %%%
    if(is_coax)
        cable_type = 'coax'; % Cable type is coaxial
        cable_type_indicator = 'V1'; % Indicator for coaxial
    else
        cable_type = 'stp'; % Cable type is shielded twisted pair
        cable_type_indicator = 'T1'; % Indicator for shielded twisted pair
    end
    test_config = sprintf('%sG_%s_%s',hdr_rate_text,'good_vs_bad',cable_type); % Test configuration name
    camera_text = sprintf('%sG+100MBASE-%s',hdr_rate_text,cable_type_indicator); % Camera text
    host_text = sprintf('100M+%sGBASE-%s',hdr_rate_text,cable_type_indicator); % Host text

    %%% Construct test cases %%%
    cable_goodIL_goodRL = cable_good; % Good insertion loss and good return loss

    cable_goodIL_badRL = cable_good; % Good insertion loss and bad return loss
    cable_goodIL_badRL.h{1,1} = cable_bad.h{1,1}; % Replace return loss with bad
    cable_goodIL_badRL.h{2,2} = cable_bad.h{2,2}; % Replace return loss with bad

    cable_badIL_goodRL = cable_bad; % Bad insertion loss and good return loss
    cable_badIL_goodRL.h{1,1} = cable_good.h{1,1}; % Replace insertion loss with good
    cable_badIL_goodRL.h{2,2} = cable_good.h{2,2}; % Replace insertion loss with good

    cable_goodIL_idealRL = cable_good; % Good insertion loss and ideal return loss
    cable_goodIL_idealRL.h{1,1} = 0; % Set return loss to ideal (0)
    cable_goodIL_idealRL.h{2,2} = 0; % Set return loss to ideal (0)

    cable_badIL_idealRL = cable_bad; % Bad insertion loss and ideal return loss
    cable_badIL_idealRL.h{1,1} = 0; % Set return loss to ideal (0)
    cable_badIL_idealRL.h{2,2} = 0; % Set return loss to ideal (0)

    cable_badIL_badRL = cable_bad; % Bad insertion loss and bad return loss

    %%% Load hybrid model %%%
    load jonsson_e241216b_hybrid_echo.mat % Load hybrid echo model again

    %%% Configure simulation variables %%%
    pcb_cutoff = 10; % PCB cutoff frequency
    t = [0:(192*ldr_symbols)-1]/22.5e9; % Time vector for PCB

    test_cases = {
        'Good IL and Good RL', cable_goodIL_goodRL,h_hybrid_echo; % Test case 1
        'Bad IL and Good RL', cable_badIL_goodRL,h_hybrid_echo; % Test case 2
        'Good IL and Bad RL', cable_goodIL_badRL,h_hybrid_echo; % Test case 3
        'Bad IL and Bad RL', cable_badIL_badRL,h_hybrid_echo; % Test case 4
        'Good IL and Ideal RL', cable_goodIL_idealRL,0; % Test case 5
        'Bad IL and Ideal RL', cable_badIL_idealRL,0; % Test case 6
    };

    %%% Loop over all test cases %%%
    N_test_cases = size(test_cases,1); % Number of test cases
    for m = 1:N_test_cases
      %%% select cable %%%
      cable_type = test_cases{m,1}; % Cable type for current test case
      cable = test_cases{m,2}; % Cable model for current test case
    
      %%% Configure the PCB impairments %%%
      pcb_config.hybrid_echo = test_cases{m,3}; % Hybrid echo for PCB
      pcb_config.cutoff = pcb_cutoff; % PCB cutoff frequency
      pcb_config.noise = '0;'; % No noise for PCB

      %%% No AFE internal noise in this simulation %%%
      afe_config.dut.noise = 'state.out = 0.000*randn(size(x));'; % No noise for DUT
      afe_config.lkp.noise = 'state.out = 0.000*randn(size(x));'; % No noise for LKP

      %%% Configure equalizer lengths %%%
      eq_config.dut.ffe_length = 1; % FFE length for DUT
      eq_config.dut.dfe_length = 1; % DFE length for DUT
      eq_config.lkp.ffe_length = 30; % FFE length for LKP
      eq_config.lkp.dfe_length = 10; % DFE length for LKP

      %%% Run test case %%%
      act = simDM_ACT_test(hdr_rate,pam_levels,cable,pcb_config,afe_config,eq_config); % Run simulation
  
      %%% Plot the HDR eye-diagram for each test run %%%
      figure
      signal_name = 'slicer input'; % Signal name for plot
      plot(t(1:act.hdr_oversampling:end)*1e9,act.lkp.pma.slicer_in,'.') % Plot slicer input
      xlabel('Time [ns]') % X-axis label
      ylabel('Signal Level') % Y-axis label
      grid on % Enable grid
      axis([0 t(end)*1e9 -1.5 1.5]) % Set axis limits
      function_text = sprintf('Test3(%s,%d)',hdr_rate_text,pam_levels); % Function text for plot
      text(t(end)*1e9*0.01,-1.5*0.85,{'(A) - Host Slicer Input',host_text,function_text},'BackgroundColor',[1,1,1]*0.95) % Add text to plot
      title(sprintf('Host slicer input for %s (%2.1fGbps PAM%d)',cable_type,hdr_rate,pam_levels)); % Title for plot
      if(print_plot)
        img_file = sprintf('%s_%s_eye_pam%d_%s.jpg',mfilename(),cable_type,pam_levels,test_config); % Image file name
        img_file = strrep(lower(img_file),' ','_'); % Replace spaces with underscores in file name
        if(is_octave)
          print(img_file) % Save plot to file
        else
          saveas(gcf,img_file,'jpg') % Save plot to file
        end
      end
    end
  
end
