function act = simDM_ACT(hdr_rate,is_coax,pcb_cutoff,pam_levels)
% simDM_ACT Simulates the Digital Modulation for IEEE 802.3dm.
% 
% This function simulates the digital modulation for IEEE 802.3dm, 
% including the effects of analog filters, PCB traces, and other 
% components. It returns a structure containing the simulation data.
%
% Inputs:
%   hdr_rate   - High data rate in Gbps (default: 10)
%   is_coax    - Boolean indicating if coaxial cable is used (default: 1)
%   pcb_cutoff - PCB cutoff frequency in MHz (default: 10)
%   pam_levels - Number of PAM levels (default: 4)
%
% Outputs:
%   act - Structure containing the simulation data
%
% This code is provided for reference to allow independent evaluation
% of the accuracy and applicability of the simulation results shared in
% IEEE 802.3dm presentations by the author.
%
% Written by Ragnar Jonsson, affiliated with Marvell Technology, Inc.
% Version 1.1, January 20th, 2025
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
% OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
% THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
% DEALINGS IN THE SOFTWARE.

  %%% Set default values for inputs if not provided %%%
  if(nargin < 1)
    hdr_rate = 10;
  end
  if(nargin < 2)
    is_coax = 1;
  end
  if(nargin < 3)
    pcb_cutoff = 10;
  end
  if(nargin < 4)
    pam_levels = 4;
  end
  
  %%% Calculate bits per sample based on PAM levels %%%
  bits_per_sample = log2(pam_levels);
  
  %%% Initialize transmission power levels %%%
  ldr_tx_power_dBm = 0;
  hdr_tx_power_dBm = 0;

  %%% Create the simulation data structure %%%
  act = link();

  %%% Set oversampling ratio for both high and low data rate (22.5GHz) %%%
  act.hdr_oversampling = 20/hdr_rate*bits_per_sample;
  act.ldr_oversampling = 192;

  %%% Emulate LDR Tx and Rx analog filters as 4th order Butterworth filters %%% 
  [b,a] = butter(4,2.5/act.ldr_oversampling);  % 293MHz cut-off frequency 
  act.lkp.afe.tx_filter.a = a;
  act.lkp.afe.tx_filter.b = b;
  act.dut.afe.rx_filter.a = a;
  act.dut.afe.rx_filter.b = b;

  %%% Emulate HDR Tx and Rx analog filters as 4th order Butterworth filters %%%
  [b,a] = butter(4,1/act.hdr_oversampling);
  act.lkp.afe.rx_filter.a = a;
  act.lkp.afe.rx_filter.b = b;
  act.dut.afe.tx_filter.a = a;
  act.dut.afe.tx_filter.b = b;

  %%% Emulate PCB trace as simple RC circuit %%%
  b_pcb = [0.25 0.25];
  a_pcb = [1 -0.5];

  %%% Emulate AC-cap as RC circuit %%%
  R = 100;
  C = 10e-9; % 10nF, per Table 149C–2
  f_s = 22.5e9;
  f_0 = 1/(C*R);
  b_cap = 2*f_s/(2*f_s + f_0)*[1 -1];
  a_cap = [1 -(2*f_s-f_0)/(2*f_s+f_0)];

  %%% Emulate PoC HP characteristics as second order Butterworth filters %%%
  [b_poc,a_poc] = butter(2,pcb_cutoff*2/22.5e3,'high');

  %%% Emulate the total PCB transfer function as product of component transfer %%%
  a = conv(a_pcb,conv(a_poc,a_cap));
  b = conv(b_pcb,conv(b_poc,b_cap));
  
  %%% Adjust transmission scale based on cable type %%%
  if(is_coax)
    tx_scale = 0.5;
  else
    tx_scale = 1;
  end
  
  %%% Set PCB filters in the simulation structure %%%
  act.lkp.pcb.tx_filter.a = a;
  act.lkp.pcb.tx_filter.b = b*tx_scale;
  act.lkp.pcb.rx_filter.a = a;
  act.lkp.pcb.rx_filter.b = b;
  act.dut.pcb = act.lkp.pcb;

  %%% Map LDR bits to Manchester code %%%
  act.lkp.pma.mapper.hold = [1 -1];

  %%% Emulate LDR DAC as Zero-order-Hold %%%
  ldr_signal_scale = sqrt((10^(ldr_tx_power_dBm/10))*100/1000)*1.15;
  act.lkp.afe.dac.hold = ones(1,act.ldr_oversampling/2)*ldr_signal_scale;

  %%% Emulate HDR DAC as Zero-order-Hold %%%
  hdr_signal_scale = sqrt((10^(hdr_tx_power_dBm/10))*100/1000)*1.15;
  hdr_signal_scale = hdr_signal_scale/sqrt(mean(([0:(pam_levels-1)]*2/(pam_levels-1)-1).^2));
  act.dut.afe.dac.hold = ones(1,act.hdr_oversampling)*hdr_signal_scale;

  %%% Configure LKP sampling and slicing %%%
  pam_text = sprintf('pam%d',pam_levels);
  act.lkp.pma.ffe_oversampling = 2;
  act.lkp.afe.adc_sampling.rate = 1/2*hdr_rate/10*act.lkp.pma.ffe_oversampling/log2(pam_levels);
  act.lkp.pma.slicer.quant = Qformat(pam_text);

end
